/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.application;

/**
 * An extension to the {@link AbstractService} that has an {@link EngineService}
 * member field and a hops-to-live value.
 * 
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id: AbstractEngineService.java 130 2006-02-18 18:27:11Z bombe $
 */
public abstract class AbstractEngineService extends AbstractService {

	/**
	 * The engine.
	 */
	protected EngineService engine;

	/**
	 * The hops-to-live value for engine operations.
	 */
	protected int hopsToLive;

	/**
	 * Trusted network name.
	 */
	protected String networkName = "freeMule";

	/**
	 * Sets the engine for this Service.
	 * 
	 * @param engine
	 *            The engine for this Service
	 */
	public void setEngine(EngineService engine) {
		this.engine = engine;
	}

	/**
	 * Sets the hops-to-live value for EngineService operations.
	 * 
	 * @param hopsToLive
	 *            The hops-to-live value
	 */
	public void setHopsToLive(int hopsToLive) {
		this.hopsToLive = hopsToLive;
	}

	/**
	 * @param networkName
	 *            The networkName to set.
	 */
	public void setNetworkName(String networkName) {
		this.networkName = networkName;
	}

	/**
	 * {@inheritDoc}
	 */
	public void init() {
		super.init();
		engine = null;
		hopsToLive = -1;
		networkName = "freeMule";
	}

	/**
	 * {@inheritDoc}
	 */
	public void destroy() {
		networkName = null;
		engine = null;
		hopsToLive = -1;
		super.destroy();
	}

}
