/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.application;

import java.util.EventListener;

/**
 * Interface for listeners that want to be notified on filelist insertion
 * events.
 * 
 * @see freemule.application.FilelistInsertService
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id: FilelistInsertListener.java 130 2006-02-18 18:27:11Z bombe $
 */
public interface FilelistInsertListener extends EventListener {

	/**
	 * Notifies listeners that the insertion of a filelist has started.
	 * 
	 * @param key
	 *            The key of the filelist
	 * @param index
	 *            The index of the filelist
	 */
	public void filelistInsertStarted(String key, int index);

	/**
	 * Notifies listeners that the insertion of a filelist has started.
	 * 
	 * @param key
	 *            The key of the filelist
	 * @param index
	 *            The index of the filelist
	 * @param exception
	 *            The exception that caused insertion to fail
	 */
	public void filelistInsertFailed(String key, int index, Exception exception);

	/**
	 * Notifies listeners that the insertion of a filelist has completed
	 * successfully.
	 * 
	 * @param key
	 *            The key of the filelist
	 * @param index
	 *            The index of the filelist
	 */
	public void filelistInsertComplete(String key, int index);

}
