/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.application;

import java.util.Properties;

import freemule.util.Hex;

/**
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id: UploadFile.java 130 2006-02-18 18:27:11Z bombe $
 */
public class UploadFile extends FreenetFile {

	private boolean inserted;
	private boolean[] blockInserted;

	/**
	 * 
	 */
	public UploadFile() {
		super();
	}

	/**
	 * @param freenetFile
	 */
	public UploadFile(FreenetFile freenetFile) {
		super(freenetFile);
		if (freenetFile instanceof UploadFile) {
			UploadFile uploadFile = (UploadFile) freenetFile;
			inserted = uploadFile.inserted;
			System.arraycopy(uploadFile.blockInserted, 0, blockInserted, 0, getBlockCount());
		}
	}

	public void setInserted(boolean inserted) {
		this.inserted = inserted;
	}

	public boolean isInserted() {
		return inserted;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public synchronized void setLength(long length) {
		super.setLength(length);
		blockInserted = new boolean[getBlockCount()];
	}

	public synchronized void setBlockInserted(int blockNumber, boolean inserted) {
		blockInserted[blockNumber] = inserted;
	}

	public synchronized boolean getBlockInserted(int blockNumber) {
		return blockInserted[blockNumber];
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Properties getProperties(String prefix) {
		Properties fileProperties = super.getProperties(prefix);
		for (int blockIndex = 0, blockCount = getBlockCount(); blockIndex < blockCount; blockIndex++) {
			String blockPrefix = prefix + ".Block." + Hex.toHex(blockIndex);
			fileProperties.setProperty(blockPrefix + ".Inserted", String.valueOf(blockInserted[blockIndex]));
		}
		return fileProperties;
	}

	public static UploadFile createFromProperties(Properties fileProperties, String prefix) {
		FreenetFile freenetFile = FreenetFile.createFromProperties(fileProperties, prefix);
		if (freenetFile == null) {
			return null;
		}
		UploadFile uploadFile = new UploadFile(freenetFile);
		for (int blockIndex = 0, blockCount = uploadFile.getBlockCount(); blockIndex < blockCount; blockIndex++) {
			String blockPrefix = prefix + ".Block." + Hex.toHex(blockIndex);
			uploadFile.setBlockInserted(blockIndex, Boolean.valueOf(fileProperties.getProperty(blockPrefix + ".Inserted")));
		}
		return uploadFile;
	}

}
