/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.application;

import java.util.EventListener;

/**
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id: UploadListener.java 130 2006-02-18 18:27:11Z bombe $
 */
public interface UploadListener extends EventListener {

	/**
	 * Notifies listeners that a new upload was added.
	 * 
	 * @param uploadFile
	 *            The upload that was added
	 */
	public void uploadAdded(UploadFile uploadFile);

	/**
	 * Notifies listeners that an upload was updated.
	 * 
	 * @param uploadFile
	 *            The upload that was updated
	 */
	public void uploadUpdated(UploadFile uploadFile);

	/**
	 * Notifies listeners that an upload was removed.
	 * 
	 * @param uploadFile
	 *            The upload that was removed
	 */
	public void uploadRemoved(UploadFile uploadFile);

	/**
	 * Notifies listeners that an upload has completed.
	 * 
	 * @param uploadFile
	 *            The upload that has completed
	 */
	public void uploadComplete(UploadFile uploadFile);

	/**
	 * Notifies listeners that the upload of a block has started. If
	 * <code>blockIndex</code> is <code>-1</code> the manifest is being
	 * uploaded.
	 * 
	 * @param uploadFile
	 *            The upload of the block
	 * @param blockIndex
	 *            The number of the block, or <code>-1</code> for the manifest
	 */
	public void blockUploadStarted(UploadFile uploadFile, int blockIndex);

	/**
	 * Notifies listeners that the upload of a block has failed. If
	 * <code>blockIndex</code> is <code>-1</code> the manifest is being
	 * uploaded.
	 * 
	 * @param uploadFile
	 *            The upload of the block
	 * @param blockIndex
	 *            The number of the block, or <code>-1</code> for the manifest
	 * @param exception
	 *            The excpetion that caused the upload to fail
	 */
	public void blockUploadFailed(UploadFile uploadFile, int blockIndex, Exception exception);

	/**
	 * Notifies listeners that the upload of a block has completed. If
	 * <code>blockIndex</code> is <code>-1</code> the manifest is being
	 * uploaded.
	 * 
	 * @param uploadFile
	 *            The upload of the block
	 * @param blockIndex
	 *            The number of the block, or <code>-1</code> for the manifest
	 */
	public void blockUploadComplete(UploadFile uploadFile, int blockIndex);

	/**
	 * Notifies listeners that uploading a file's manifest has started.
	 * 
	 * @param uploadFile
	 *            The file whose manifest is uploaded
	 */
	public void manifestUploadStarted(UploadFile uploadFile);

	/**
	 * Notifies listeners that uploading a file's manifest could not be uploaded
	 * due to an error.
	 * 
	 * @param uploadFile
	 *            The file whose manifest is uploaded
	 * @param exception
	 *            The exception that caused the upload to fail
	 */
	public void manifestUploadFailed(UploadFile uploadFile, Exception exception);

	/**
	 * Notifies listeners that uploading a file's manifest is complete.
	 * 
	 * @param uploadFile
	 *            The file whose manifest was uploaded
	 */
	public void manifestUploadComplete(UploadFile uploadFile);

}
