/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.gui;

import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.io.IOException;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import freenet.fcp.FcpClient;
import freenet.fcp.FcpClientHello;
import freenet.fcp.FcpException;
import freenet.fcp.FcpNode;

/**
 * Dialog that lets the user enter the host address and port number of her
 * freenet node.
 * 
 * @author David Roden &lt;droden@gmail.com&gt;
 * @version $Id: NodeDialog.java 130 2006-02-18 18:27:11Z bombe $
 * @license GPL
 */
public class NodeDialog extends OkayCancelDialog {

	private String nodeAddress;
	private int nodePort;

	private JTextField nodeAddressField;
	private JTextField nodePortField;

	private Action testAction;

	public NodeDialog(JFrame frame) {
		super(frame, "Node Settings");

		nodeAddressField = new JTextField(15);
		nodePortField = new JTextField(5);
		testAction = new AbstractAction("Test Node") {

			public void actionPerformed(ActionEvent actionEvent) {
				testNode();
			}
		};

		JPanel contentPane = new JPanel(new LineLayout(12));
		JLabel nodePropertiesLabel = new JLabel("<html><b>Freenet Node Properties</b></html>");
		contentPane.add(nodePropertiesLabel);

		JPanel addressPanel = new JPanel(new GridBagLayout());
		contentPane.add(addressPanel);
		JLabel nodeAddressLabel = new JLabel("Node Address:");
		nodeAddressLabel.setFont(nodeAddressLabel.getFont().deriveFont(Font.PLAIN));
		JLabel nodePortLabel = new JLabel("Node Port:");
		nodePortLabel.setFont(nodePortLabel.getFont().deriveFont(Font.PLAIN));
		addressPanel.add(nodeAddressLabel, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0, GridBagConstraints.LINE_START, GridBagConstraints.NONE, new Insets(0, 24, 0, 0), 0, 0));
		addressPanel.add(nodeAddressField, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0, GridBagConstraints.LINE_START, GridBagConstraints.HORIZONTAL, new Insets(0, 6, 0, 0), 0, 0));
		addressPanel.add(nodePortLabel, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0, GridBagConstraints.LINE_START, GridBagConstraints.NONE, new Insets(6, 24, 0, 0), 0, 0));
		addressPanel.add(nodePortField, new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0, GridBagConstraints.LINE_START, GridBagConstraints.HORIZONTAL, new Insets(6, 6, 0, 0), 0, 0));
		addressPanel.add(new JButton(testAction), new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0, GridBagConstraints.LINE_START, GridBagConstraints.NONE, new Insets(6, 24, 0, 0), 0, 0));

		setContentPane(contentPane);
		pack();
	}

	/**
	 * @return Returns the nodeAddress.
	 */
	public String getNodeAddress() {
		return nodeAddress;
	}

	/**
	 * @param nodeAddress
	 *            The nodeAddress to set.
	 */
	public void setNodeAddress(String nodeAddress) {
		this.nodeAddress = nodeAddress;
		nodeAddressField.setText(nodeAddress);
	}

	/**
	 * @return Returns the nodePort.
	 */
	public int getNodePort() {
		return nodePort;
	}

	/**
	 * @param nodePort
	 *            The nodePort to set.
	 */
	public void setNodePort(int nodePort) {
		this.nodePort = nodePort;
		nodePortField.setText(String.valueOf(nodePort));
	}

	public void testNode() {
		if (checkNodePort()) {
			FcpNode node = new FcpNode(nodeAddressField.getText(), Integer.parseInt(nodePortField.getText()));
			FcpClient client = new FcpClient(node);
			FcpClientHello hello = new FcpClientHello();
			String message = "";
			int messageType = JOptionPane.ERROR_MESSAGE;
			try {
				client.execute(hello);
				messageType = JOptionPane.INFORMATION_MESSAGE;
				message = "Node is: " + hello.getNode();
			} catch (FcpException e) {
				message = "FCP Failed: " + e.getMessage();
			} catch (IOException e) {
				message = "Node not found: " + e.getMessage();
			}
			JOptionPane.showMessageDialog(this, message, "Node Test", messageType);
		} else {
			JOptionPane.showMessageDialog(this, "Port is invalid!", "Error", JOptionPane.ERROR_MESSAGE);
		}
	}

	@Override
	protected void cancelAction() {
		setVisible(false);
	}

	@Override
	protected void okayAction() {
		nodeAddress = nodeAddressField.getText();
		try {
			nodePort = Integer.parseInt(nodePortField.getText().trim());
		} catch (NumberFormatException nfe1) {
			JOptionPane.showMessageDialog(this, "Port is invalid!", "Error", JOptionPane.ERROR_MESSAGE);
			return;
		}
		setVisible(false);
	}

	private boolean checkNodePort() {
		String nodePortText = nodePortField.getText();
		int port = -1;
		try {
			port = Integer.parseInt(nodePortText);
		} catch (NumberFormatException nfe1) {
			return false;
		}
		return (port > 0) && (port < 65536);
	}

}
