/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.gui;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.swing.table.AbstractTableModel;

/**
 * A {@link javax.swing.table.TableModel} for the filenames of a
 * {@link freemule.gui.SearchResultTableModel.ResultFile} and the respective
 * counts.
 * 
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id: SearchResultFilenameTableModel.java,v 1.1 2005/12/05 00:22:08
 *          bombe Exp $
 */
public class SearchResultFilenameTableModel extends AbstractTableModel {

	/**
	 * The column names.
	 */
	private static final String[] COLUMN_NAMES = new String[] { "Filename", "Count" };

	/**
	 * The column classes.
	 */
	private static final Class[] COLUMN_CLASSES = new Class[] { String.class, Integer.class };

	/**
	 * The filename to show.
	 */
	private String[] filenames;
	/**
	 * The count of a filename. The number at a specific index <i>n</i> is the
	 * count for the filename at index <i>n</i>.
	 */
	private Integer[] counts;

	/**
	 * Creates a {@link javax.swing.table.TableModel} for filenames and counts.
	 * 
	 * @param filenames
	 *            The filename to show
	 */
	public SearchResultFilenameTableModel(String[] filenames) {
		Arrays.sort(filenames);
		String lastFilename = "";
		int currentCount = 0;
		List<String> tempFilenames = new ArrayList<String>();
		List<Integer> tempCount = new ArrayList<Integer>();
		for (String filename: filenames) {
			if (!filename.equals(lastFilename)) {
				if (!"".equals(lastFilename)) {
					tempFilenames.add(lastFilename);
					tempCount.add(++currentCount);
					currentCount = 0;
				}
			} else {
				currentCount++;
			}
			lastFilename = filename;
		}
		if (!"".equals(lastFilename)) {
			tempFilenames.add(lastFilename);
			tempCount.add(++currentCount);
		}
		this.filenames = tempFilenames.toArray(new String[tempFilenames.size()]);
		this.counts = tempCount.toArray(new Integer[tempCount.size()]);
	}

	/**
	 * Returns the number of rows.
	 * 
	 * @return The number of rows
	 */
	public int getRowCount() {
		return filenames.length;
	}

	/**
	 * Returns the number of columns.
	 * 
	 * @return The number of columns
	 */
	public int getColumnCount() {
		return COLUMN_NAMES.length;
	}

	/**
	 * Returns the {@link Class} of a column.
	 * 
	 * @return The {@link Class} of a column
	 */
	@Override
	public Class< ? > getColumnClass(int columnIndex) {
		return COLUMN_CLASSES[columnIndex];
	}

	/**
	 * Returns the name of a column.
	 * 
	 * @return The name of a column
	 */
	@Override
	public String getColumnName(int column) {
		return COLUMN_NAMES[column];
	}

	/**
	 * Returns the value of the table model at the specified coordinates.
	 * 
	 * @param rowIndex
	 *            The index of the row
	 * @param columnIndex
	 *            The index of the column
	 * @return The value at the specified coordinates
	 */
	public Object getValueAt(int rowIndex, int columnIndex) {
		switch (columnIndex) {
		case 0:
			return filenames[rowIndex];
		case 1:
			return counts[rowIndex];
		}
		return null;
	}

}
