/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.EventObject;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableCellEditor;

import freemule.application.SharedDirectory;


/**
 * @author David Roden &lt;droden@gmail.com&gt;
 * @version $Id$
 * @license GPL
 */
public class SharedDirectoriesDialog extends OkayCancelDialog implements ListSelectionListener, MouseListener {

	
	private Action addAction;
	private Action editAction;
	private Action deleteAction;
	private JPopupMenu contextMenu;
	private SharedDirectory[] sharedDirectories;
	private JTable sharedDirectoriesTable;
	private SharedDirectoriesModel sharedDirectoriesTableModel;
	
	public SharedDirectoriesDialog(JFrame owner) {
		super(owner, "Shared Directories");
		createActions();
		initialize();
		pack();
	}
	
	public void setSharedDirectories(SharedDirectory[] sharedDirectories) {
		this.sharedDirectories = new SharedDirectory[sharedDirectories.length];
		SharedDirectory[] tempDirectories = new SharedDirectory[sharedDirectories.length];
		System.arraycopy(sharedDirectories, 0, this.sharedDirectories, 0, sharedDirectories.length);
		System.arraycopy(sharedDirectories, 0, tempDirectories, 0, sharedDirectories.length);
		sharedDirectoriesTableModel.setSharedDirectories(tempDirectories);
	}
	
	public SharedDirectory[] getSharedDirectories() {
		return sharedDirectories;
	}
	 
	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void okayAction() {
		sharedDirectories = sharedDirectoriesTableModel.getSharedDirectories();
		super.okayAction();
	}
	
	private void createActions() {
		contextMenu = new JPopupMenu("Shared Directory");
		
		addAction = new AbstractAction("Add...") {
			public void actionPerformed(ActionEvent actionEvent) {
				actionAdd();
			}
		};
		addAction.putValue(Action.SMALL_ICON, IconLoader.loadIcon("/img/shared-add.png"));
		
		editAction = new AbstractAction("Edit") {
			public void actionPerformed(ActionEvent actionEvent) {
				actionEdit();
			}
		};
		editAction.putValue(Action.SMALL_ICON, IconLoader.loadIcon("/img/shared-edit.png"));
		editAction.setEnabled(false);
		
		deleteAction = new AbstractAction("Delete") {
			public void actionPerformed(ActionEvent actionEvent) {
				actionDelete();
			}
		};
		deleteAction.putValue(Action.SMALL_ICON, IconLoader.loadIcon("/img/shared-delete.png"));
		deleteAction.setEnabled(false);
		
		contextMenu.add(addAction);
		contextMenu.add(editAction);
		contextMenu.add(deleteAction);
	}
	
	private void initialize() {
		JComponent contentPane = new JPanel(new BorderLayout(12, 12));
		
		sharedDirectoriesTableModel = new SharedDirectoriesModel();
		sharedDirectoriesTable = new JTable(sharedDirectoriesTableModel);
		sharedDirectoriesTable.addMouseListener(this);
		sharedDirectoriesTable.getSelectionModel().addListSelectionListener(this);
		sharedDirectoriesTable.setDefaultEditor(String.class, new SharedDirectoryCellEditor());
		
		JLabel sharedDirectoriesLabel = new JLabel("<html><b>Shared directories:</b></html>");
		sharedDirectoriesLabel.setDisplayedMnemonic('s');
		sharedDirectoriesLabel.setLabelFor(sharedDirectoriesTable);
		contentPane.add(sharedDirectoriesLabel, BorderLayout.NORTH);
		contentPane.add(new JScrollPane(sharedDirectoriesTable), BorderLayout.CENTER);
		
		JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.TRAILING, 12, 12));
		buttonPanel.setBorder(new EmptyBorder(-12, -12, -12, -12));
		buttonPanel.add(new JButton(addAction));
		buttonPanel.add(new JButton(editAction));
		buttonPanel.add(new JButton(deleteAction));
		contentPane.add(buttonPanel, BorderLayout.SOUTH);
		
		setContentPane(contentPane);
	}
	
	protected void actionAdd() {
		String directory = browseForDirectory("");
		if (directory != null) {
			sharedDirectoriesTableModel.addSharedDirectory(directory, false);
		}
	}
	
	protected void actionEdit() {
		int row = sharedDirectoriesTable.getSelectedRow();
		String directory = browseForDirectory((String) sharedDirectoriesTableModel.getValueAt(row, 0));
		if (directory != null) {
			sharedDirectoriesTableModel.setValueAt(directory, row, 0);
		}
	}
	
	protected void actionDelete() {
		int[] rows = sharedDirectoriesTable.getSelectedRows();
		for (int rowIndex = rows.length - 1; rowIndex >= 0; rowIndex--) {
			sharedDirectoriesTableModel.deleteSharedDirectory(rows[rowIndex]);
		}
	}
	
	protected String browseForDirectory(String directory) {
		JFileChooser fileChooser = new JFileChooser(directory);
		fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		int answer = fileChooser.showDialog(this, "Choose");
		if (answer == JFileChooser.APPROVE_OPTION) {
			return fileChooser.getSelectedFile().getPath();
		}
		return null;
	}

	//
	// INTERFACE MouseListener
	//
	
	/**
	 * {@inheritDoc}
	 */
	public void mouseClicked(MouseEvent e) {
		if (e.isPopupTrigger()) {
			contextMenu.show(sharedDirectoriesTable, e.getX(), e.getY());
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public void mousePressed(MouseEvent e) {
		if (e.isPopupTrigger()) {
			contextMenu.show(sharedDirectoriesTable, e.getX(), e.getY());
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public void mouseReleased(MouseEvent e) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void mouseEntered(MouseEvent e) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void mouseExited(MouseEvent e) {
	}
	
	//
	// INTERFACE ListSelectionListener
	//

	/**
	 * {@inheritDoc}
	 */
	public void valueChanged(ListSelectionEvent e) {
		int selectionCount = sharedDirectoriesTable.getSelectedRowCount();
		editAction.setEnabled(selectionCount == 1);
		deleteAction.setEnabled(selectionCount > 0);
	}

	/**
	 * @author David Roden &lt;droden@gmail.com&gt;
	 * @version $Id$
	 * @license GPL
	 */
	public class SharedDirectoryCellEditor extends JPanel implements TableCellEditor {

		private JTextField textField;
		private Action browseAction;
		private List<CellEditorListener> listeners = new ArrayList<CellEditorListener>();
		
		public SharedDirectoryCellEditor() {
			textField = new JTextField();
			textField.setBorder(new EmptyBorder(0, 1, 0, 0));
			browseAction = new AbstractAction("...") {
				public void actionPerformed(ActionEvent actionEvent) {
					actionBrowse();
				}
			};
			setLayout(new BorderLayout());
			add(textField, BorderLayout.CENTER);
			add(new JButton(browseAction), BorderLayout.LINE_END);
		}
		
		protected void actionBrowse() {
			String directory = browseForDirectory(textField.getText());
			if (directory != null) {
				textField.setText(directory);
			}
			stopCellEditing();
		}
		
		/**
		 * {@inheritDoc}
		 */
		public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
			textField.setText((String) value);
			return this;
		}

		/**
		 * {@inheritDoc}
		 */
		public Object getCellEditorValue() {
			return textField.getText();
		}

		/**
		 * {@inheritDoc}
		 */
		public boolean isCellEditable(EventObject anEvent) {
			return true;
		}

		/**
		 * {@inheritDoc}
		 */
		public boolean shouldSelectCell(EventObject anEvent) {
			return true;
		}

		/**
		 * {@inheritDoc}
		 */
		public boolean stopCellEditing() {
			fireEditingStopped();
			return true;
		}

		/**
		 * {@inheritDoc}
		 */
		public void cancelCellEditing() {
			fireEditingCancelled();
		}
		
		private void fireEditingStopped() {
			final ChangeEvent changeEvent = new ChangeEvent(this);
			for (final CellEditorListener cellEditorListener: listeners) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						cellEditorListener.editingStopped(changeEvent);
					}						
				});
			}
		}

		private void fireEditingCancelled() {
			final ChangeEvent changeEvent = new ChangeEvent(this);
			for (final CellEditorListener cellEditorListener: listeners) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						cellEditorListener.editingCanceled(changeEvent);
					}						
				});
			}
		}

		/**
		 * {@inheritDoc}
		 */
		public void addCellEditorListener(CellEditorListener cellEditorListener) {
			listeners.add(cellEditorListener);
		}

		/**
		 * {@inheritDoc}
		 */
		public void removeCellEditorListener(CellEditorListener cellEditorListener) {
			listeners.remove(cellEditorListener);
		}

	}

}
