/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.gui;

import java.awt.BorderLayout;

import javax.swing.Box;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;

import freemule.application.SharedFile;

/**
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id: SharedFilePropertiesDialog.java,v 1.1 2005/11/28 22:50:39 bombe
 *          Exp $
 */
public class SharedFilePropertiesDialog extends OkayCancelDialog {

	private JTextField commentField;
	private JComboBox ratingBox;

	private String comment;
	private int rating;

	public SharedFilePropertiesDialog(JFrame mainFrame, String filename) {
		super(mainFrame, "Comment and Rating for " + filename);
		initializeComponents();
	}

	/**
	 * 
	 */
	private void initializeComponents() {
		JPanel contentPane = new JPanel(new BorderLayout(5, 5));
		contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));

		Box lines = Box.createVerticalBox();
		contentPane.add(lines, BorderLayout.CENTER);
		JPanel firstLine = new JPanel(new BorderLayout(5, 5));
		lines.add(firstLine);
		firstLine.add(new JLabel("Comment:"), BorderLayout.LINE_START);
		commentField = new JTextField(40);
		firstLine.add(commentField, BorderLayout.CENTER);
		lines.add(Box.createVerticalStrut(5));
		JPanel secondLine = new JPanel(new BorderLayout(5, 5));
		lines.add(secondLine);
		secondLine.add(new JLabel("Rating:"), BorderLayout.LINE_START);
		ratingBox = new JComboBox(SharedFile.RATING);
		secondLine.add(ratingBox, BorderLayout.CENTER);
		lines.add(Box.createVerticalStrut(5));
		setContentPane(contentPane);
		pack();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void okayAction() {
		comment = commentField.getText();
		rating = ratingBox.getSelectedIndex();
		setVisible(false);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void cancelAction() {
		setVisible(false);
	}

	/**
	 * @return Returns the comment.
	 */
	public String getComment() {
		return comment;
	}

	/**
	 * @param comment
	 *            The comment to set.
	 */
	public void setComment(String comment) {
		this.comment = comment;
		commentField.setText(comment);
	}

	/**
	 * @return Returns the rating.
	 */
	public int getRating() {
		return rating;
	}

	/**
	 * @param rating
	 *            The rating to set.
	 */
	public void setRating(int rating) {
		this.rating = rating;
		ratingBox.setSelectedIndex(rating);
	}

}
