/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

import freemule.application.SharedFile;
import freemule.application.SharedFileService;
import freemule.util.NumberFormatter;

/**
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id: SharedFilesPanel.java 130 2006-02-18 18:27:11Z bombe $
 */
public class SharedFilesPanel extends JPanel {

	private Action editCommentAction;

	private JFrame parentFrame;
	protected JTable sharedFilesTable;
	private TableSorter sharedFilesSorter;
	private SharedFileTableModel sharedFilesModel;
	private JLabel infoField;
	protected JPopupMenu popupMenu;

	private SharedFileService sharedFileService;

	public SharedFilesPanel(JFrame parentFrame) {
		super(new BorderLayout(5, 5));
		this.parentFrame = parentFrame;
		initializeActions();
		initializeComponents();
	}

	private void initializeActions() {
		editCommentAction = new AbstractAction("Edit Comment") {

			public void actionPerformed(ActionEvent actionEvent) {
				actionEditComment();
			}
		};
	}

	private void initializeComponents() {
		sharedFilesModel = new SharedFileTableModel();
		sharedFilesSorter = new TableSorter(sharedFilesModel);
		sharedFilesTable = new JTable(sharedFilesSorter);
		sharedFilesSorter.addTableModelListener(new TableModelListener() {

			public void tableChanged(TableModelEvent tableModelEvent) {
				updateInfoField();
			}
		});
		sharedFilesSorter.setTableHeader(sharedFilesTable.getTableHeader());
		sharedFilesTable.addMouseListener(new MouseAdapter() {

			public void mouseClicked(MouseEvent mouseEvent) {
				JTable sourceTable = (JTable) mouseEvent.getSource();
				int selectedRowIndex = sourceTable.getSelectedRow();
				if ((selectedRowIndex != -1) && mouseEvent.isPopupTrigger()) {
					popupMenu.show(sharedFilesTable, mouseEvent.getX(), mouseEvent.getY());
				}
				if (mouseEvent.getClickCount() == 2) {
					actionEditComment();
				}
			}

			public void mousePressed(MouseEvent mouseEvent) {
				JTable sourceTable = (JTable) mouseEvent.getSource();
				int selectedRowIndex = sourceTable.getSelectedRow();
				if ((selectedRowIndex != -1) && mouseEvent.isPopupTrigger()) {
					popupMenu.show(sharedFilesTable, mouseEvent.getX(), mouseEvent.getY());
				}
			}
		});
		infoField = new JLabel();
		infoField.setBorder(new CompoundBorder(new EtchedBorder(EtchedBorder.LOWERED), new EmptyBorder(1, 1, 1, 1)));
		popupMenu = new JPopupMenu();
		popupMenu.add(editCommentAction);

		setBorder(new EmptyBorder(5, 5, 5, 5));

		add(new JScrollPane(sharedFilesTable), BorderLayout.CENTER);
		add(infoField, BorderLayout.PAGE_END);
		updateInfoField();
	}

	protected void actionEditComment() {
		int selectedRow = sharedFilesTable.getSelectedRow();
		SharedFile sharedFile = (SharedFile) sharedFilesSorter.getValueAt(selectedRow, -1);
		SharedFilePropertiesDialog dialog = new SharedFilePropertiesDialog(parentFrame, sharedFile.getName());
		String oldComment = sharedFile.getComment().trim();
		int oldRating = sharedFile.getRating();
		dialog.setComment(oldComment);
		dialog.setRating(oldRating);
		dialog.setVisible(true);
		String newComment = dialog.getComment();
		int newRating = dialog.getRating();
		if (!newComment.equals(oldComment) || (newRating != oldRating)) {
			sharedFile.setComment(newComment);
			sharedFile.setRating(newRating);
			sharedFilesSorter.fireTableRowsUpdated(selectedRow, selectedRow);
			sharedFileService.setChanged(true);
		}
	}

	protected void updateInfoField() {
		SharedFile[] sharedFiles = sharedFilesModel.getSharedFiles();
		int fileCount = 0;
		long sizeSum = 0;
		if ((sharedFiles != null) && (sharedFiles.length != 0)) {
			fileCount = sharedFiles.length;
			for (SharedFile sharedFile: sharedFiles) {
				sizeSum += sharedFile.getLength();
			}
		}
		infoField.setText("Number of Files: " + fileCount + ", Number of Bytes: " + sizeSum + " Bytes (" + NumberFormatter.format(sizeSum) + ")");
	}

	public void setSharedFileService(SharedFileService sharedFileService) {
		this.sharedFileService = sharedFileService;
		sharedFilesModel.setSharedFileService(sharedFileService);
		sharedFilesModel.fireTableDataChanged();
	}

}
