/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.main;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.StringTokenizer;

import freemule.application.AnnouncementService;
import freemule.application.ClientService;
import freemule.application.ClipboardService;
import freemule.application.DiscoveryService;
import freemule.application.DownloadService;
import freemule.application.EngineService;
import freemule.application.FilelistCollectorService;
import freemule.application.FilelistInsertService;
import freemule.application.FilelistService;
import freemule.application.FreeMule;
import freemule.application.SharedDirectory;
import freemule.application.SharedFileService;
import freemule.application.UploadRequestDiscoveryService;
import freemule.application.UploadRequestInsertService;
import freemule.application.UploadRequestService;
import freemule.application.UploadService;

/**
 * @author David Roden &lt;droden@gmail.com&gt;
 * @version $Id: Configuration.java 130 2006-02-18 18:27:11Z bombe $
 */
public class Configuration {

	private Properties configItems = new Properties();
	private LoggingPreferences loggingPreferences;

	public LoggingPreferences getLoggingPreferences() {
		if (loggingPreferences != null) {
			return loggingPreferences;
		}
		loggingPreferences = new LoggingPreferences();
		loggingPreferences.setLogfile(configItems.getProperty("Logging.Logfile", "stdout"));
		String serviceNames = configItems.getProperty("Logging.Services");
		if (serviceNames != null) {
			StringTokenizer serviceNameTokenizer = new StringTokenizer(serviceNames, ",");
			while (serviceNameTokenizer.hasMoreTokens()) {
				String serviceName = serviceNameTokenizer.nextToken();
				String eventNames = configItems.getProperty("Logging.Service." + serviceName);
				StringTokenizer eventNameTokenizer = new StringTokenizer(eventNames, ",");
				LoggingOptions loggingOptions = new LoggingOptions(serviceName);
				while (eventNameTokenizer.hasMoreElements()) {
					String eventName = eventNameTokenizer.nextToken();
					String eventPropertyName = "Logging.Event." + serviceName + "." + eventName;
					int loggingValue = Integer.parseInt(configItems.getProperty(eventPropertyName));
					loggingOptions.setLogging(eventName, (loggingValue & 1) == 1, (loggingValue & 2) == 2);
				}
				loggingPreferences.addServiceLoggingOptions(loggingOptions);
				String groupNames = configItems.getProperty("Logging.Groups." + serviceName);
				StringTokenizer groupNameTokenizer = new StringTokenizer(groupNames, ",");
				while (groupNameTokenizer.hasMoreTokens()) {
					String groupName = groupNameTokenizer.nextToken();
					String groupEventNames = configItems.getProperty("Logging.Group." + serviceName + "." + groupName + ".EventNames");
					List<String> groupEventNameList = new ArrayList<String>();
					StringTokenizer groupEventNameTokenizer = new StringTokenizer(groupEventNames, ",");
					while (groupEventNameTokenizer.hasMoreTokens()) {
						String groupEventName = groupEventNameTokenizer.nextToken();
						groupEventNameList.add(groupEventName);
					}
					loggingOptions.addLoggingGroup(groupName, groupEventNameList.toArray(new String[groupEventNameList.size()]));
				}
			}
		}
		if (loggingPreferences.getServiceNames().length == 0) {
			loggingPreferences.setLogfile("stdout");
			LoggingOptions freeMuleLoggingOptions = new LoggingOptions(FreeMule.NAME);
			freeMuleLoggingOptions.addLoggingGroup("Main Service Events", new String[] { "Service Started", "Service Aborted", "Service Stopped" });
			freeMuleLoggingOptions.setLogging("Service Started", true, true);
			freeMuleLoggingOptions.setLogging("Service Aborted", true, true);
			freeMuleLoggingOptions.setLogging("Service Stopped", true, true);
			freeMuleLoggingOptions.addLoggingGroup("Service Thread Events", new String[] { "Thread Started", "Thread Updated", "Thread Aborted", "Thread Stopped" });
			freeMuleLoggingOptions.setLogging("Thread Started", true, true);
			freeMuleLoggingOptions.setLogging("Thread Updated", true, true);
			freeMuleLoggingOptions.setLogging("Thread Aborted", true, true);
			freeMuleLoggingOptions.setLogging("Thread Stopped", true, true);
			loggingPreferences.addServiceLoggingOptions(freeMuleLoggingOptions);

			LoggingOptions announcementLoggingOptions = new LoggingOptions(AnnouncementService.NAME);
			announcementLoggingOptions.addLoggingGroup("Main Service Events", new String[] { "Service Started", "Service Aborted", "Service Stopped" });
			announcementLoggingOptions.setLogging("Service Started", true, true);
			announcementLoggingOptions.setLogging("Service Aborted", true, true);
			announcementLoggingOptions.setLogging("Service Stopped", true, true);
			announcementLoggingOptions.addLoggingGroup("Service Thread Events", new String[] { "Thread Started", "Thread Updated", "Thread Aborted", "Thread Stopped" });
			announcementLoggingOptions.setLogging("Thread Started", true, true);
			announcementLoggingOptions.setLogging("Thread Updated", true, true);
			announcementLoggingOptions.setLogging("Thread Aborted", true, true);
			announcementLoggingOptions.setLogging("Thread Stopped", true, true);
			announcementLoggingOptions.addLoggingGroup("Announcement Events", new String[] { "Insert Started", "Insert Failed", "Insert Completed" });
			announcementLoggingOptions.setLogging("Insert Started", true, true);
			announcementLoggingOptions.setLogging("Insert Failed", true, true);
			announcementLoggingOptions.setLogging("Insert Completed", true, true);
			loggingPreferences.addServiceLoggingOptions(announcementLoggingOptions);

			LoggingOptions clientServiceLoggingOptions = new LoggingOptions(ClientService.NAME);
			clientServiceLoggingOptions.addLoggingGroup("Main Service Events", new String[] { "Service Started", "Service Aborted", "Service Stopped" });
			clientServiceLoggingOptions.setLogging("Service Started", true, true);
			clientServiceLoggingOptions.setLogging("Service Aborted", true, true);
			clientServiceLoggingOptions.setLogging("Service Stopped", true, true);
			clientServiceLoggingOptions.addLoggingGroup("Service Thread Events", new String[] { "Thread Started", "Thread Updated", "Thread Aborted", "Thread Stopped" });
			clientServiceLoggingOptions.setLogging("Thread Started", true, true);
			clientServiceLoggingOptions.setLogging("Thread Updated", true, true);
			clientServiceLoggingOptions.setLogging("Thread Aborted", true, true);
			clientServiceLoggingOptions.setLogging("Thread Stopped", true, true);
			clientServiceLoggingOptions.addLoggingGroup("Remote Client Events", new String[] { "Remote Client Added", "Remote Client Updated", "Remote Client Removed" });
			clientServiceLoggingOptions.setLogging("Remote Client Added", true, true);
			clientServiceLoggingOptions.setLogging("Remote Client Updated", true, true);
			clientServiceLoggingOptions.setLogging("Remote Client Removed", false, true);
			loggingPreferences.addServiceLoggingOptions(clientServiceLoggingOptions);

			LoggingOptions clipboardServiceLoggingOptions = new LoggingOptions(ClipboardService.NAME);
			clipboardServiceLoggingOptions.addLoggingGroup("Main Service Events", new String[] { "Service Started", "Service Aborted", "Service Stopped" });
			clipboardServiceLoggingOptions.setLogging("Service Started", true, true);
			clipboardServiceLoggingOptions.setLogging("Service Aborted", true, true);
			clipboardServiceLoggingOptions.setLogging("Service Stopped", true, true);
			clipboardServiceLoggingOptions.addLoggingGroup("Service Thread Events", new String[] { "Thread Started", "Thread Updated", "Thread Aborted", "Thread Stopped" });
			clipboardServiceLoggingOptions.setLogging("Thread Started", true, true);
			clipboardServiceLoggingOptions.setLogging("Thread Updated", true, true);
			clipboardServiceLoggingOptions.setLogging("Thread Aborted", true, true);
			clipboardServiceLoggingOptions.setLogging("Thread Stopped", true, true);
			clipboardServiceLoggingOptions.addLoggingGroup("Clipboard Events", new String[] { "URL Found" });
			clipboardServiceLoggingOptions.setLogging("URL Found", true, true);
			loggingPreferences.addServiceLoggingOptions(clipboardServiceLoggingOptions);

			LoggingOptions discoveryServiceLoggingOptions = new LoggingOptions(DiscoveryService.NAME);
			discoveryServiceLoggingOptions.addLoggingGroup("Main Service Events", new String[] { "Service Started", "Service Aborted", "Service Stopped" });
			discoveryServiceLoggingOptions.setLogging("Service Started", true, true);
			discoveryServiceLoggingOptions.setLogging("Service Aborted", true, true);
			discoveryServiceLoggingOptions.setLogging("Service Stopped", true, true);
			discoveryServiceLoggingOptions.addLoggingGroup("Service Thread Events", new String[] { "Thread Started", "Thread Updated", "Thread Aborted", "Thread Stopped" });
			discoveryServiceLoggingOptions.setLogging("Thread Started", true, true);
			discoveryServiceLoggingOptions.setLogging("Thread Updated", true, true);
			discoveryServiceLoggingOptions.setLogging("Thread Aborted", true, true);
			discoveryServiceLoggingOptions.setLogging("Thread Stopped", true, true);
			discoveryServiceLoggingOptions.addLoggingGroup("Announcement Discovery Events", new String[] { "Discovery Started", "Discovery Failed", "Discovery Completed" });
			discoveryServiceLoggingOptions.setLogging("Discovery Started", true, true);
			discoveryServiceLoggingOptions.setLogging("Discovery Failed", true, true);
			discoveryServiceLoggingOptions.setLogging("Discovery Completed", true, true);
			loggingPreferences.addServiceLoggingOptions(discoveryServiceLoggingOptions);

			LoggingOptions downloadServiceLoggingOptions = new LoggingOptions(DownloadService.NAME);
			downloadServiceLoggingOptions.addLoggingGroup("Main Service Events", new String[] { "Service Started", "Service Aborted", "Service Stopped" });
			downloadServiceLoggingOptions.setLogging("Service Started", true, true);
			downloadServiceLoggingOptions.setLogging("Service Aborted", true, true);
			downloadServiceLoggingOptions.setLogging("Service Stopped", true, true);
			downloadServiceLoggingOptions.addLoggingGroup("Service Thread Events", new String[] { "Thread Started", "Thread Updated", "Thread Aborted", "Thread Stopped" });
			downloadServiceLoggingOptions.setLogging("Thread Started", true, true);
			downloadServiceLoggingOptions.setLogging("Thread Updated", true, true);
			downloadServiceLoggingOptions.setLogging("Thread Aborted", true, true);
			downloadServiceLoggingOptions.setLogging("Thread Stopped", true, true);
			downloadServiceLoggingOptions.addLoggingGroup("Downloads Loading Events", new String[] { "Downloads Loading Started", "Downloads Loading Failed", "Downloads Loading Completed" });
			downloadServiceLoggingOptions.setLogging("Downloads Loading Started", true, true);
			downloadServiceLoggingOptions.setLogging("Downloads Loading Failed", true, true);
			downloadServiceLoggingOptions.setLogging("Downloads Loading Completed", true, true);
			downloadServiceLoggingOptions.addLoggingGroup("Downloads Saving Events", new String[] { "Downloads Saving Started", "Downloads Saving Failed", "Downloads Saving Completed" });
			downloadServiceLoggingOptions.setLogging("Downloads Saving Started", true, true);
			downloadServiceLoggingOptions.setLogging("Downloads Saving Failed", true, true);
			downloadServiceLoggingOptions.setLogging("Downloads Saving Completed", true, true);
			downloadServiceLoggingOptions.addLoggingGroup("Download Loading Events", new String[] { "Download Loading Started", "Download Loading Failed", "Download Loading Completed" });
			downloadServiceLoggingOptions.setLogging("Download Loading Started", true, true);
			downloadServiceLoggingOptions.setLogging("Download Loading Failed", true, true);
			downloadServiceLoggingOptions.setLogging("Download Loading Completed", true, true);
			downloadServiceLoggingOptions.addLoggingGroup("Download Saving Events", new String[] { "Download Saving Started", "Download Saving Failed", "Download Saving Completed" });
			downloadServiceLoggingOptions.setLogging("Download Saving Started", true, true);
			downloadServiceLoggingOptions.setLogging("Download Saving Failed", true, true);
			downloadServiceLoggingOptions.setLogging("Download Saving Completed", true, true);
			downloadServiceLoggingOptions.addLoggingGroup("Download Main Events", new String[] { "Download Added", "Download Removed", "Download Stopped", "Download Started", "Download Completing", "Download Completion Failed", "Download Completed" });
			downloadServiceLoggingOptions.setLogging("Download Added", true, true);
			downloadServiceLoggingOptions.setLogging("Download Removed", true, true);
			downloadServiceLoggingOptions.setLogging("Download Stopped", true, true);
			downloadServiceLoggingOptions.setLogging("Download Started", true, true);
			downloadServiceLoggingOptions.setLogging("Download Completing", true, true);
			downloadServiceLoggingOptions.setLogging("Download Completion Failed", true, true);
			downloadServiceLoggingOptions.setLogging("Download Completed", true, true);
			downloadServiceLoggingOptions.addLoggingGroup("Manifest Events", new String[] { "Manifest Download Started", "Manifest Download Failed", "Manifest Download Completed" });
			downloadServiceLoggingOptions.setLogging("Manifest Download Started", true, true);
			downloadServiceLoggingOptions.setLogging("Manifest Download Failed", true, true);
			downloadServiceLoggingOptions.setLogging("Manifest Download Completed", true, true);
			downloadServiceLoggingOptions.addLoggingGroup("Block Events", new String[] { "Block Download Started", "Block Download Failed", "Block Download Completed" });
			downloadServiceLoggingOptions.setLogging("Block Download Started", true, true);
			downloadServiceLoggingOptions.setLogging("Block Download Failed", true, true);
			downloadServiceLoggingOptions.setLogging("Block Download Completed", true, true);
			downloadServiceLoggingOptions.addLoggingGroup("Block Saving Events", new String[] { "Block Saving Started", "Block Saving Failed", "Block Saving Completed" });
			downloadServiceLoggingOptions.setLogging("Block Saving Started", true, true);
			downloadServiceLoggingOptions.setLogging("Block Saving Failed", true, true);
			downloadServiceLoggingOptions.setLogging("Block Saving Completed", true, true);
			downloadServiceLoggingOptions.addLoggingGroup("Segment Events", new String[] { "Segment Decodable" });
			downloadServiceLoggingOptions.setLogging("Segment Decodable", true, true);
			loggingPreferences.addServiceLoggingOptions(downloadServiceLoggingOptions);

			LoggingOptions engineServiceLoggingOptions = new LoggingOptions(EngineService.NAME);
			engineServiceLoggingOptions.addLoggingGroup("Main Service Events", new String[] { "Service Started", "Service Aborted", "Service Stopped" });
			engineServiceLoggingOptions.setLogging("Service Started", true, true);
			engineServiceLoggingOptions.setLogging("Service Aborted", true, true);
			engineServiceLoggingOptions.setLogging("Service Stopped", true, true);
			engineServiceLoggingOptions.addLoggingGroup("Service Thread Events", new String[] { "Thread Started", "Thread Updated", "Thread Aborted", "Thread Stopped" });
			engineServiceLoggingOptions.setLogging("Thread Started", true, true);
			engineServiceLoggingOptions.setLogging("Thread Updated", true, true);
			engineServiceLoggingOptions.setLogging("Thread Aborted", true, true);
			engineServiceLoggingOptions.setLogging("Thread Stopped", true, true);
			loggingPreferences.addServiceLoggingOptions(engineServiceLoggingOptions);

			LoggingOptions filelistCollectorServiceLoggingOptions = new LoggingOptions(FilelistCollectorService.NAME);
			filelistCollectorServiceLoggingOptions.addLoggingGroup("Main Service Events", new String[] { "Service Started", "Service Aborted", "Service Stopped" });
			filelistCollectorServiceLoggingOptions.setLogging("Service Started", true, true);
			filelistCollectorServiceLoggingOptions.setLogging("Service Aborted", true, true);
			filelistCollectorServiceLoggingOptions.setLogging("Service Stopped", true, true);
			filelistCollectorServiceLoggingOptions.addLoggingGroup("Service Thread Events", new String[] { "Thread Started", "Thread Updated", "Thread Aborted", "Thread Stopped" });
			filelistCollectorServiceLoggingOptions.setLogging("Thread Started", true, true);
			filelistCollectorServiceLoggingOptions.setLogging("Thread Updated", true, true);
			filelistCollectorServiceLoggingOptions.setLogging("Thread Aborted", true, true);
			filelistCollectorServiceLoggingOptions.setLogging("Thread Stopped", true, true);
			filelistCollectorServiceLoggingOptions.addLoggingGroup("Filelist Collecting Events", new String[] { "Collecting Started", "Collecting Failed", "Collecting Completed" });
			filelistCollectorServiceLoggingOptions.setLogging("Collecting Started", true, true);
			filelistCollectorServiceLoggingOptions.setLogging("Collecting Failed", true, true);
			filelistCollectorServiceLoggingOptions.setLogging("Collecting Completed", true, true);
			loggingPreferences.addServiceLoggingOptions(filelistCollectorServiceLoggingOptions);

			LoggingOptions filelistInsertServiceLoggingOptions = new LoggingOptions(FilelistInsertService.NAME);
			filelistInsertServiceLoggingOptions.addLoggingGroup("Main Service Events", new String[] { "Service Started", "Service Aborted", "Service Stopped" });
			filelistInsertServiceLoggingOptions.setLogging("Service Started", true, true);
			filelistInsertServiceLoggingOptions.setLogging("Service Aborted", true, true);
			filelistInsertServiceLoggingOptions.setLogging("Service Stopped", true, true);
			filelistInsertServiceLoggingOptions.addLoggingGroup("Service Thread Events", new String[] { "Thread Started", "Thread Updated", "Thread Aborted", "Thread Stopped" });
			filelistInsertServiceLoggingOptions.setLogging("Thread Started", true, true);
			filelistInsertServiceLoggingOptions.setLogging("Thread Updated", true, true);
			filelistInsertServiceLoggingOptions.setLogging("Thread Aborted", true, true);
			filelistInsertServiceLoggingOptions.setLogging("Thread Stopped", true, true);
			filelistInsertServiceLoggingOptions.addLoggingGroup("Filelist Insert Events", new String[] { "Insert Started", "Insert Failed", "Insert Completed" });
			filelistInsertServiceLoggingOptions.setLogging("Insert Started", true, true);
			filelistInsertServiceLoggingOptions.setLogging("Insert Failed", true, true);
			filelistInsertServiceLoggingOptions.setLogging("Insert Completed", true, true);
			loggingPreferences.addServiceLoggingOptions(filelistInsertServiceLoggingOptions);

			LoggingOptions filelistServiceLoggingOptions = new LoggingOptions(FilelistService.NAME);
			filelistServiceLoggingOptions.addLoggingGroup("Main Service Events", new String[] { "Service Started", "Service Aborted", "Service Stopped" });
			filelistServiceLoggingOptions.setLogging("Service Started", true, true);
			filelistServiceLoggingOptions.setLogging("Service Aborted", true, true);
			filelistServiceLoggingOptions.setLogging("Service Stopped", true, true);
			filelistServiceLoggingOptions.addLoggingGroup("Service Thread Events", new String[] { "Thread Started", "Thread Updated", "Thread Aborted", "Thread Stopped" });
			filelistServiceLoggingOptions.setLogging("Thread Started", true, true);
			filelistServiceLoggingOptions.setLogging("Thread Updated", true, true);
			filelistServiceLoggingOptions.setLogging("Thread Aborted", true, true);
			filelistServiceLoggingOptions.setLogging("Thread Stopped", true, true);
			filelistServiceLoggingOptions.addLoggingGroup("Filelist Events", new String[] { "Filelist Added", "Filelist Updated", "Filelist Removed" });
			filelistServiceLoggingOptions.setLogging("Filelist Added", true, true);
			filelistServiceLoggingOptions.setLogging("Filelist Updated", true, true);
			filelistServiceLoggingOptions.setLogging("Filelist Removed", true, true);
			loggingPreferences.addServiceLoggingOptions(filelistServiceLoggingOptions);

			LoggingOptions sharedFileServiceLoggingOptions = new LoggingOptions(SharedFileService.NAME);
			sharedFileServiceLoggingOptions.addLoggingGroup("Main Service Events", new String[] { "Service Started", "Service Aborted", "Service Stopped" });
			sharedFileServiceLoggingOptions.setLogging("Service Started", true, true);
			sharedFileServiceLoggingOptions.setLogging("Service Aborted", true, true);
			sharedFileServiceLoggingOptions.setLogging("Service Stopped", true, true);
			sharedFileServiceLoggingOptions.addLoggingGroup("Service Thread Events", new String[] { "Thread Started", "Thread Updated", "Thread Aborted", "Thread Stopped" });
			sharedFileServiceLoggingOptions.setLogging("Thread Started", true, true);
			sharedFileServiceLoggingOptions.setLogging("Thread Updated", true, true);
			sharedFileServiceLoggingOptions.setLogging("Thread Aborted", true, true);
			sharedFileServiceLoggingOptions.setLogging("Thread Stopped", true, true);
			sharedFileServiceLoggingOptions.addLoggingGroup("Shared Files Loading Events", new String[] { "Loading Started", "Loading Failed", "Loading Completed" });
			sharedFileServiceLoggingOptions.setLogging("Loading Started", true, true);
			sharedFileServiceLoggingOptions.setLogging("Loading Failed", true, true);
			sharedFileServiceLoggingOptions.setLogging("Loading Completed", true, true);
			sharedFileServiceLoggingOptions.addLoggingGroup("Shared Files Saving Events", new String[] { "Saving Started", "Saving Failed", "Saving Completed" });
			sharedFileServiceLoggingOptions.setLogging("Saving Started", true, true);
			sharedFileServiceLoggingOptions.setLogging("Saving Failed", true, true);
			sharedFileServiceLoggingOptions.setLogging("Saving Completed", true, true);
			sharedFileServiceLoggingOptions.addLoggingGroup("Directories Scan Events", new String[] { "Directories Scan Started", "Directories Scan Failed", "Directories Scan Completed" });
			sharedFileServiceLoggingOptions.setLogging("Directories Scan Started", true, true);
			sharedFileServiceLoggingOptions.setLogging("Directories Scan Failed", true, true);
			sharedFileServiceLoggingOptions.setLogging("Directories Scan Completed", true, true);
			sharedFileServiceLoggingOptions.addLoggingGroup("Directory Scan Events", new String[] { "Directory Scan Started", "Directory Scan Failed", "Directory Scan Completed" });
			sharedFileServiceLoggingOptions.setLogging("Directory Scan Started", true, true);
			sharedFileServiceLoggingOptions.setLogging("Directory Scan Failed", true, true);
			sharedFileServiceLoggingOptions.setLogging("Directory Scan Completed", true, true);
			sharedFileServiceLoggingOptions.addLoggingGroup("File Hashing Events", new String[] { "Hashing Started", "Hashing Progress", "Hashing Failed", "Hashing Completed" });
			sharedFileServiceLoggingOptions.setLogging("Hashing Started", true, true);
			sharedFileServiceLoggingOptions.setLogging("Hashing Progress", true, true);
			sharedFileServiceLoggingOptions.setLogging("Hashing Failed", true, true);
			sharedFileServiceLoggingOptions.setLogging("Hashing Completed", true, true);
			sharedFileServiceLoggingOptions.addLoggingGroup("List Events", new String[] { "List Changed", "Directory Unshared", "File Unshared" });
			sharedFileServiceLoggingOptions.setLogging("List Changed", true, true);
			sharedFileServiceLoggingOptions.setLogging("Directory Unshared", true, true);
			sharedFileServiceLoggingOptions.setLogging("File Unshared", true, true);
			loggingPreferences.addServiceLoggingOptions(sharedFileServiceLoggingOptions);

			LoggingOptions uploadRequestDiscoveryServiceLoggingOptions = new LoggingOptions(UploadRequestDiscoveryService.NAME);
			uploadRequestDiscoveryServiceLoggingOptions.addLoggingGroup("Main Service Events", new String[] { "Service Started", "Service Aborted", "Service Stopped" });
			uploadRequestDiscoveryServiceLoggingOptions.setLogging("Service Started", true, true);
			uploadRequestDiscoveryServiceLoggingOptions.setLogging("Service Aborted", true, true);
			uploadRequestDiscoveryServiceLoggingOptions.setLogging("Service Stopped", true, true);
			uploadRequestDiscoveryServiceLoggingOptions.addLoggingGroup("Service Thread Events", new String[] { "Thread Started", "Thread Updated", "Thread Aborted", "Thread Stopped" });
			uploadRequestDiscoveryServiceLoggingOptions.setLogging("Thread Started", true, true);
			uploadRequestDiscoveryServiceLoggingOptions.setLogging("Thread Updated", true, true);
			uploadRequestDiscoveryServiceLoggingOptions.setLogging("Thread Aborted", true, true);
			uploadRequestDiscoveryServiceLoggingOptions.setLogging("Thread Stopped", true, true);
			uploadRequestDiscoveryServiceLoggingOptions.addLoggingGroup("Upload Request Discovery Events", new String[] { "Discovery Started", "Discovery Failed", "Discovery Completed" });
			uploadRequestDiscoveryServiceLoggingOptions.setLogging("Discovery Started", true, true);
			uploadRequestDiscoveryServiceLoggingOptions.setLogging("Discovery Failed", true, true);
			uploadRequestDiscoveryServiceLoggingOptions.setLogging("Discovery Completed", true, true);
			loggingPreferences.addServiceLoggingOptions(uploadRequestDiscoveryServiceLoggingOptions);

			LoggingOptions uploadRequestInsertServiceLoggingOptions = new LoggingOptions(UploadRequestInsertService.NAME);
			uploadRequestInsertServiceLoggingOptions.addLoggingGroup("Main Service Events", new String[] { "Service Started", "Service Aborted", "Service Stopped" });
			uploadRequestInsertServiceLoggingOptions.setLogging("Service Started", true, true);
			uploadRequestInsertServiceLoggingOptions.setLogging("Service Aborted", true, true);
			uploadRequestInsertServiceLoggingOptions.setLogging("Service Stopped", true, true);
			uploadRequestInsertServiceLoggingOptions.addLoggingGroup("Service Thread Events", new String[] { "Thread Started", "Thread Updated", "Thread Aborted", "Thread Stopped" });
			uploadRequestInsertServiceLoggingOptions.setLogging("Thread Started", true, true);
			uploadRequestInsertServiceLoggingOptions.setLogging("Thread Updated", true, true);
			uploadRequestInsertServiceLoggingOptions.setLogging("Thread Aborted", true, true);
			uploadRequestInsertServiceLoggingOptions.setLogging("Thread Stopped", true, true);
			uploadRequestInsertServiceLoggingOptions.addLoggingGroup("Upload Request Events", new String[] { "Upload Request Started", "Upload Request Failed", "Upload Request Completed" });
			uploadRequestInsertServiceLoggingOptions.setLogging("Upload Request Started", true, true);
			uploadRequestInsertServiceLoggingOptions.setLogging("Upload Request Failed", true, true);
			uploadRequestInsertServiceLoggingOptions.setLogging("Upload Request Completed", true, true);
			loggingPreferences.addServiceLoggingOptions(uploadRequestInsertServiceLoggingOptions);

			LoggingOptions uploadRequestServiceLoggingOptions = new LoggingOptions(UploadRequestService.NAME);
			uploadRequestServiceLoggingOptions.addLoggingGroup("Main Service Events", new String[] { "Service Started", "Service Aborted", "Service Stopped" });
			uploadRequestServiceLoggingOptions.setLogging("Service Started", true, true);
			uploadRequestServiceLoggingOptions.setLogging("Service Aborted", true, true);
			uploadRequestServiceLoggingOptions.setLogging("Service Stopped", true, true);
			uploadRequestServiceLoggingOptions.addLoggingGroup("Service Thread Events", new String[] { "Thread Started", "Thread Updated", "Thread Aborted", "Thread Stopped" });
			uploadRequestServiceLoggingOptions.setLogging("Thread Started", true, true);
			uploadRequestServiceLoggingOptions.setLogging("Thread Updated", true, true);
			uploadRequestServiceLoggingOptions.setLogging("Thread Aborted", true, true);
			uploadRequestServiceLoggingOptions.setLogging("Thread Stopped", true, true);
			uploadRequestServiceLoggingOptions.addLoggingGroup("Upload Request Events", new String[] { "Upload Request Added", "Upload Request Updated", "Upload Request Removed" });
			uploadRequestServiceLoggingOptions.setLogging("Upload Request Added", true, true);
			uploadRequestServiceLoggingOptions.setLogging("Upload Request Updated", true, true);
			uploadRequestServiceLoggingOptions.setLogging("Upload Request Removed", true, true);
			loggingPreferences.addServiceLoggingOptions(uploadRequestServiceLoggingOptions);

			LoggingOptions uploadServiceLoggingOptions = new LoggingOptions(UploadService.NAME);
			uploadServiceLoggingOptions.addLoggingGroup("Main Service Events", new String[] { "Service Started", "Service Aborted", "Service Stopped" });
			uploadServiceLoggingOptions.setLogging("Service Started", true, true);
			uploadServiceLoggingOptions.setLogging("Service Aborted", true, true);
			uploadServiceLoggingOptions.setLogging("Service Stopped", true, true);
			uploadServiceLoggingOptions.addLoggingGroup("Service Thread Events", new String[] { "Thread Started", "Thread Updated", "Thread Aborted", "Thread Stopped" });
			uploadServiceLoggingOptions.setLogging("Thread Started", true, true);
			uploadServiceLoggingOptions.setLogging("Thread Updated", true, true);
			uploadServiceLoggingOptions.setLogging("Thread Aborted", true, true);
			uploadServiceLoggingOptions.setLogging("Thread Stopped", true, true);
			uploadServiceLoggingOptions.addLoggingGroup("Upload Events", new String[] { "Upload Added", "Upload Updated", "Upload Removed" });
			uploadServiceLoggingOptions.setLogging("Upload Added", true, true);
			uploadServiceLoggingOptions.setLogging("Upload Updated", true, true);
			uploadServiceLoggingOptions.setLogging("Upload Removed", true, true);
			uploadServiceLoggingOptions.addLoggingGroup("Block Upload Events", new String[] { "Block Upload Started", "Block Upload Failed", "Block Upload Completed" });
			uploadServiceLoggingOptions.setLogging("Block Upload Started", true, true);
			uploadServiceLoggingOptions.setLogging("Block Upload Failed", true, true);
			uploadServiceLoggingOptions.setLogging("Block Upload Completed", true, true);
			uploadServiceLoggingOptions.addLoggingGroup("Manifest Upload Events", new String[] { "Manifest Upload Started", "Manifest Upload Failed", "Manifest Upload Completed" });
			uploadServiceLoggingOptions.setLogging("Manifest Upload Started", true, true);
			uploadServiceLoggingOptions.setLogging("Manifest Upload Failed", true, true);
			uploadServiceLoggingOptions.setLogging("Manifest Upload Completed", true, true);
			loggingPreferences.addServiceLoggingOptions(uploadServiceLoggingOptions);
		}
		return loggingPreferences;
	}

	public void setLoggingPreferences(LoggingPreferences loggingPreferences) {
		this.loggingPreferences = loggingPreferences;
	}

	private void saveLoggingPreferences() {
		String[] serviceNames = loggingPreferences.getServiceNames();
		String serviceNameList = "";
		for (String serviceName: serviceNames) {
			if (serviceNameList.length() > 0) {
				serviceNameList += ",";
			}
			serviceNameList += serviceName;
			LoggingOptions serviceLoggingOptions = loggingPreferences.getServiceLoggingOptions(serviceName);
			String[] groupNames = serviceLoggingOptions.getGroups();
			String groupNameList = "";
			String eventNameList = "";
			for (String groupName: groupNames) {
				if (groupNameList.length() > 0) {
					groupNameList += ",";
				}
				groupNameList += groupName;
				String[] eventNames = serviceLoggingOptions.getGroupEventNames(groupName);
				String groupEventNameList = "";
				for (String eventName: eventNames) {
					if (eventNameList.length() > 0) {
						eventNameList += ",";
					}
					eventNameList += eventName;
					if (groupEventNameList.length() > 0) {
						groupEventNameList += ",";
					}
					groupEventNameList += eventName;
					int loggingValue = serviceLoggingOptions.getGUILogging(eventName) ? 1 : 0;
					loggingValue += serviceLoggingOptions.getFileLogging(eventName) ? 2 : 0;
					configItems.setProperty("Logging.Event." + serviceName + "." + eventName, String.valueOf(loggingValue));
				}
				configItems.setProperty("Logging.Group." + serviceName + "." + groupName + ".EventNames", groupEventNameList);
			}
			configItems.setProperty("Logging.Groups." + serviceName, groupNameList);
			configItems.setProperty("Logging.Service." + serviceName, eventNameList);
		}
		configItems.setProperty("Logging.Services", serviceNameList);
		if (loggingPreferences.getLogfile() != null) {
			configItems.setProperty("Logging.Logfile", loggingPreferences.getLogfile());
		}
	}

	public int getFilelistIndex() {
		return Integer.parseInt(configItems.getProperty("FilelistIndex", "0"));
	}

	public void setFilelistIndex(int index) {
		configItems.setProperty("FilelistIndex", String.valueOf(index));
	}

	/**
	 * @return Returns the entropy.
	 */
	public String getEntropy() {
		return configItems.getProperty("Entropy");
	}

	/**
	 * @param entropy
	 *            The entropy to set.
	 */
	public void setEntropy(String entropy) {
		configItems.setProperty("Entropy", entropy);
	}

	/**
	 * @return Returns the incomingDirectory.
	 */
	public String getIncomingDirectory() {
		return configItems.getProperty("IncomingDirectory");
	}

	/**
	 * @param incomingDirectory
	 *            The incomingDirectory to set.
	 */
	public void setIncomingDirectory(String incomingDirectory) {
		configItems.setProperty("IncomingDirectory", incomingDirectory);
	}

	/**
	 * @return Returns the nodeAddress.
	 */
	public String getNodeAddress() {
		return configItems.getProperty("NodeAddress");
	}

	/**
	 * @param nodeAddress
	 *            The nodeAddress to set.
	 */
	public void setNodeAddress(String nodeAddress) {
		configItems.setProperty("NodeAddress", nodeAddress);
	}

	/**
	 * @return Returns the nodePort.
	 */
	public int getNodePort() {
		try {
			return Integer.parseInt(configItems.getProperty("NodePort"));
		} catch (NumberFormatException nfe1) {
			return 0;
		}
	}

	/**
	 * @param nodePort
	 *            The nodePort to set.
	 */
	public void setNodePort(int nodePort) {
		configItems.setProperty("NodePort", String.valueOf(nodePort));
	}

	/**
	 * @return Returns the privateKey.
	 */
	public String getPrivateKey() {
		return configItems.getProperty("PrivateKey");
	}

	/**
	 * @param privateKey
	 *            The privateKey to set.
	 */
	public void setPrivateKey(String privateKey) {
		configItems.setProperty("PrivateKey", privateKey);
	}

	/**
	 * @return Returns the publicKey.
	 */
	public String getPublicKey() {
		return configItems.getProperty("PublicKey");
	}

	/**
	 * @param publicKey
	 *            The publicKey to set.
	 */
	public void setPublicKey(String publicKey) {
		configItems.setProperty("PublicKey", publicKey);
	}

	public String getNetworkName() {
		return configItems.getProperty("NetworkName", "freeMule");
	}

	public void setNetworkName(String networkName) {
		configItems.setProperty("NetworkName", networkName);
	}

	/**
	 * @return Returns the tempDirectory.
	 */
	public String getTempDirectory() {
		return configItems.getProperty("TempDirectory");
	}

	/**
	 * @param tempDirectory
	 *            The tempDirectory to set.
	 */
	public void setTempDirectory(String tempDirectory) {
		configItems.setProperty("TempDirectory", tempDirectory);
	}

	/**
	 * @return Returns the username.
	 */
	public String getUsername() {
		return configItems.getProperty("Username");
	}

	/**
	 * @param username
	 *            The username to set.
	 */
	public void setUsername(String username) {
		configItems.setProperty("Username", username);
	}

	public void setAnnouncementThreads(int announcementThreads) {
		configItems.setProperty("AnnouncementThreads", String.valueOf(announcementThreads));
	}

	public int getAnnouncementThreads() {
		return Integer.parseInt(configItems.getProperty("AnnouncementThreads", "3"));
	}

	public void setHopsToLive(int hopsToLive) {
		configItems.setProperty("HopsToLive", String.valueOf(hopsToLive));
	}

	public int getHopsToLive() {
		return Integer.parseInt(configItems.getProperty("HopsToLive", "25"));
	}

	public void setUploadThreads(int uploads) {
		configItems.setProperty("UploadThreads", String.valueOf(uploads));
	}

	public int getUploadThreads() {
		return Integer.parseInt(configItems.getProperty("UploadThreads", "3"));
	}

	public void setDownloadThreads(int downloads) {
		configItems.setProperty("DownloadThreads", String.valueOf(downloads));
	}

	public int getDownloadThreads() {
		return Integer.parseInt(configItems.getProperty("DownloadThreads", "3"));
	}

	public SharedDirectory[] getSharedDirectories() {
		List<SharedDirectory> sharedDirectories = new ArrayList<SharedDirectory>();
		int index = 0;
		while (configItems.containsKey("SharedDirectory." + index + ".Directory")) {
			String directory = configItems.getProperty("SharedDirectory." + index + ".Directory");
			boolean recursive = Boolean.valueOf(configItems.getProperty("SharedDirectory." + index + ".Recursive")).booleanValue();
			SharedDirectory sharedDirectory = new SharedDirectory(directory, recursive);
			sharedDirectories.add(sharedDirectory);
			index++;
		}
		return sharedDirectories.toArray(new SharedDirectory[sharedDirectories.size()]);
	}

	public void setSharedDirectories(SharedDirectory[] sharedDirectories) {
		// delete all previous entries
		List<String> itemsToRemove = new ArrayList<String>();
		Iterator keys = configItems.keySet().iterator();
		while (keys.hasNext()) {
			String key = (String) keys.next();
			if (key.startsWith("SharedDirectory.")) {
				itemsToRemove.add(key);
			}
		}
		for (String key: itemsToRemove) {
			configItems.remove(key);
		}
		// now enter new shared directories
		int index = 0;
		for (SharedDirectory sharedDirectory: sharedDirectories) {
			configItems.setProperty("SharedDirectory." + index + ".Directory", sharedDirectory.getDirectory());
			configItems.setProperty("SharedDirectory." + index + ".Recursive", String.valueOf(sharedDirectory.isRecursive()));
			index++;
		}
	}

	public void load() {
		String homeDir = System.getProperty("user.home") + "/.freeMule";
		File homeFile = new File(homeDir);
		if (!homeFile.exists()) {
			homeFile.mkdir();
		}
		File configFile = new File(homeFile, "config");
		if (configFile.exists()) {
			try {
				configItems.clear();
				configItems.load(new FileInputStream(configFile));
			} catch (IOException e) {
			}
		} else {
			// empty config!
		}

		String nodeAddress = getNodeAddress();
		if (nodeAddress == null) {
			setNodeAddress("localhost");
		}

		int nodePort = getNodePort();
		if ((nodePort < 1) || (nodePort > 65535)) {
			setNodePort(8481);
		}

		String username = getUsername();
		if (username == null) {
			String n = "";
			for (int i = 0; i < 20; i++) {
				n += (char) (65 + ((int) (Math.random() * 26)) + ((int) (Math.random() * 2) * 32));
			}
			setUsername(n);
		}

		String incomingDirectory = getIncomingDirectory();
		if (incomingDirectory == null) {
			incomingDirectory = System.getProperty("user.home") + "/.freeMule/incoming";
			File inDir = new File(incomingDirectory);
			inDir.mkdirs();
			setIncomingDirectory(incomingDirectory);
		}

		String tempDirectory = getTempDirectory();
		if (tempDirectory == null) {
			tempDirectory = System.getProperty("user.home") + "/.freeMule/temp";
			File tempDir = new File(tempDirectory);
			tempDir.mkdirs();
			setTempDirectory(tempDirectory);
		}

		String networkName = getNetworkName();
		if (networkName == null) {
			setNetworkName("freeMule");
		}
		getLoggingPreferences();
		save();
	}

	public void save() {
		File freeMuleDirectory = new File(System.getProperty("user.home"), ".freeMule");
		if (!freeMuleDirectory.exists()) {
			freeMuleDirectory.mkdir();
		}
		File configFile = new File(freeMuleDirectory, "config");
		saveLoggingPreferences();
		try {
			configItems.store(new FileOutputStream(configFile), "freeMule config file -- do not change!");
		} catch (IOException e) {
			System.err.println("Could not write config to ~/.freeMule/config!");
		}
	}

}
