/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.main;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class LoggingOptions {

	private String serviceName;
	private List<String> groupNames;
	private Map<String, String[]> groupEventNames;
	private Map<String, String> eventNameGroups;
	private Map<String, Boolean> logToGuiEvents;
	private Map<String, Boolean> logToFileEvents;

	public LoggingOptions(String serviceName) {
		this.serviceName = serviceName;
		this.groupNames = new ArrayList<String>();
		this.groupEventNames = new HashMap<String, String[]>();
		this.eventNameGroups = new HashMap<String, String>();
		this.logToGuiEvents = new HashMap<String, Boolean>();
		this.logToFileEvents = new HashMap<String, Boolean>();
	}

	public void addLoggingGroup(String groupName, String[] eventNames) {
		groupNames.add(groupName);
		groupEventNames.put(groupName, eventNames);
		for (String eventName: eventNames) {
			eventNameGroups.put(eventName, groupName);
		}
	}

	public String getServiceName() {
		return serviceName;
	}

	public String[] getGroups() {
		return groupNames.toArray(new String[groupNames.size()]);
	}

	public String[] getGroupEventNames(String groupName) {
		return groupEventNames.get(groupName);
	}

	public void setLogging(String eventName, boolean logToGui, boolean logToFile) {
		setGUILogging(eventName, logToGui);
		setFileLogging(eventName, logToFile);
	}

	public void setGUILogging(String eventName, boolean logToGui) {
		logToGuiEvents.put(eventName, logToGui);

	}

	public void setFileLogging(String eventName, boolean logToFile) {
		logToFileEvents.put(eventName, logToFile);

	}

	public boolean getGUILogging(String eventName) {
		return logToGuiEvents.get(eventName);
	}

	public boolean getFileLogging(String eventName) {
		return logToFileEvents.get(eventName);
	}

}