/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.util;

import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;


/**
 * @author David Roden &lt;droden@gmail.com&gt;
 * @version $Id$
 * @license GPL
 */
public class SimpleXML {
	
	private List<SimpleXML> children = new ArrayList<SimpleXML>();
	private String name = null;
	private String value = null;
	
	public SimpleXML() {
		super();
	}
	
	public SimpleXML(String name) {
		this.name = name;
	}
	
	public SimpleXML getNode(String nodeName) {
		for (int index = 0, count = children.size(); index < count; index++) {
			if (children.get(index).name.equals(nodeName)) {
				return children.get(index);
			}
		}
		return null;
	}
	
	public SimpleXML getNode(String[] nodeNames) {
		SimpleXML node = this;
		for (String nodeName: nodeNames) {
			node = node.getNode(nodeName);
		}
		return node;
	}
	
	public SimpleXML[] getNodes(String nodeName) {
		List<SimpleXML> resultList = new ArrayList<SimpleXML>();
		for (SimpleXML child: children) {
			if (child.name.equals(nodeName)) {
				resultList.add(child);
			}
		}
		return resultList.toArray(new SimpleXML[resultList.size()]);
	}
	
	public SimpleXML append(String nodeName) {
		SimpleXML newNode = new SimpleXML(nodeName);
		children.add(newNode);
		return newNode;
	}
	
	public SimpleXML append(String nodeName, String nodeValue) {
		return append(nodeName).setValue(nodeValue);
	}
	
	public SimpleXML setValue(String nodeValue) {
		value = nodeValue;
		return this;
	}
	
	public String getValue() {
		return value;
	}
	
	public Document getDocument() {
		DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
		try {
			DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
			Document document = documentBuilder.newDocument();
			Element rootElement = document.createElement(name);
			document.appendChild(rootElement);
			addChildren(rootElement);
			return document;
		} catch (ParserConfigurationException e) {
		}
		return null;
	}
	
	private void addChildren(Element rootElement) {
		for (SimpleXML child: children) {
			Element childElement = rootElement.getOwnerDocument().createElement(child.name);
			rootElement.appendChild(childElement);
			if (child.value != null) {
				Text childText = rootElement.getOwnerDocument().createTextNode(child.value);
				childElement.appendChild(childText);
			} else {
				child.addChildren(childElement);
			}
		}
	}
	
	public static SimpleXML fromDocument(Document document) {
		SimpleXML xmlDocument = new SimpleXML();
		document.normalizeDocument();
		return addDocumentChildren(xmlDocument, document);
	}
	
	public static SimpleXML addDocumentChildren(SimpleXML xmlDocument, Node document) {
		NodeList childNodes = document.getChildNodes();
		for (int childIndex = 0, childCount = childNodes.getLength(); childIndex < childCount; childIndex++) {
			Node childNode = childNodes.item(childIndex);
			if ((childNode.getChildNodes().getLength() == 1) && (childNode.getFirstChild().getNodeName().equals("#text"))) {
				xmlDocument.append(childNode.getNodeName(), childNode.getFirstChild().getNodeValue());
			} else {
				SimpleXML newXML = xmlDocument.append(childNode.getNodeName());
				addDocumentChildren(newXML, childNode);
			}
		}
		return xmlDocument;
	}

}
