/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.util;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.nio.charset.Charset;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.SAXException;

import freemule.application.FreenetFile;

/**
 * @author David Roden &lt;droden@gmail.com&gt;
 * @version $Id$
 * @license GPL
 */
public class XML {

	private static DocumentBuilderFactory documentBuilderFactory = null;
	private static DocumentBuilder documentBuilder = null;
	private static TransformerFactory transformerFactory = null;

	private static DocumentBuilderFactory getDocumentBuilderFactory() {
		if (documentBuilderFactory != null) {
			return documentBuilderFactory;
		}
		documentBuilderFactory = DocumentBuilderFactory.newInstance();
		return documentBuilderFactory;
	}

	private static DocumentBuilder getDocumentBuilder() {
		if (documentBuilder != null) {
			return documentBuilder;
		}
		try {
			documentBuilder = getDocumentBuilderFactory().newDocumentBuilder();
		} catch (ParserConfigurationException e) {
		}
		return documentBuilder;
	}

	private static TransformerFactory getTransformerFactory() {
		if (transformerFactory != null) {
			return transformerFactory;
		}
		transformerFactory = TransformerFactory.newInstance();
		return transformerFactory;
	}

	public static Document createDocument() {
		return getDocumentBuilder().newDocument();
	}

	public static Document createDocumentFromFile(FreenetFile freenetFile) {
		Document fileDocument = createDocument();

		Element sharedFileElement = fileDocument.createElement("file");
		fileDocument.appendChild(sharedFileElement);

		Element nameElement = fileDocument.createElement("name");
		sharedFileElement.appendChild(nameElement);

		Text nameText = fileDocument.createTextNode(freenetFile.getName());
		nameElement.appendChild(nameText);

		Element lengthElement = fileDocument.createElement("length");
		sharedFileElement.appendChild(lengthElement);

		Text lengthText = fileDocument.createTextNode(Hex.toHex(freenetFile.getLength()));
		lengthElement.appendChild(lengthText);

		Element blockListElement = fileDocument.createElement("blocklist");
		sharedFileElement.appendChild(blockListElement);

		for (int blockIndex = 0, blockCount = freenetFile.getBlockCount(); blockIndex < blockCount; blockIndex++) {
			Element blockElement = fileDocument.createElement("block");
			blockListElement.appendChild(blockElement);

			Text blockText = fileDocument.createTextNode(freenetFile.getBlockKey(blockIndex));
			blockElement.appendChild(blockText);
		}

		// create xml tree
		return fileDocument;
	}
	
	public static void fillFileFromDocument(FreenetFile freenetFile, Document fileDocument) {
		NodeList documentChildList = fileDocument.getChildNodes();
		for (int documentChildIndex = 0, documentChildCount = documentChildList.getLength(); documentChildIndex < documentChildCount; documentChildIndex++) {
			Node documentChild = documentChildList.item(documentChildIndex);
			if ("file".equals(documentChild.getNodeName())) {
				NodeList fileNodeChildList = documentChild.getChildNodes();
				for (int fileNodeChildIndex = 0, fileNodeChildCount = fileNodeChildList.getLength(); fileNodeChildIndex < fileNodeChildCount; fileNodeChildIndex++) {
					Node fileNodeChild = fileNodeChildList.item(fileNodeChildIndex);
					if ("name".equals(fileNodeChild.getNodeName())) {
						NodeList nameNodeChildList = fileNodeChild.getChildNodes();
						for (int nameNodeChildIndex = 0, nameNodeChildCount = nameNodeChildList.getLength(); nameNodeChildIndex < nameNodeChildCount; nameNodeChildIndex++) {
							Node nameNodeChild = nameNodeChildList.item(nameNodeChildIndex);
							if ("#text".equals(nameNodeChild.getNodeName())) {
								freenetFile.setName(nameNodeChild.getNodeValue());
							}
						}
					} else if ("length".equals(fileNodeChild.getNodeName())) {
						NodeList lengthNodeChildList = fileNodeChild.getChildNodes();
						for (int lengthNodeChildIndex = 0, lengthNodeChildCount = lengthNodeChildList.getLength(); lengthNodeChildIndex < lengthNodeChildCount; lengthNodeChildIndex++) {
							Node lengthNodeChild = lengthNodeChildList.item(lengthNodeChildIndex);
							if ("#text".equals(lengthNodeChild.getNodeName())) {
								freenetFile.setLength(Hex.toLong(lengthNodeChild.getNodeValue()));
							}
						}
					} else if ("blocklist".equals(fileNodeChild.getNodeName())) {
						NodeList blockListChildNodeList = fileNodeChild.getChildNodes();
						int blockIndex = 0;
						for (int blockListChildNodeIndex = 0, blockListChildNodeCount = blockListChildNodeList.getLength(); blockListChildNodeIndex < blockListChildNodeCount; blockListChildNodeIndex++) {
							Node blockListChildNode = blockListChildNodeList.item(blockListChildNodeIndex);
							if ("block".equals(blockListChildNode.getNodeName())) {
								NodeList blockChildNodeList = blockListChildNode.getChildNodes();
								for (int blockChildNodeIndex = 0, blockChildNodeCount = blockChildNodeList.getLength(); blockChildNodeIndex < blockChildNodeCount; blockChildNodeIndex++) {
									Node blockChildNode = blockChildNodeList.item(blockChildNodeIndex);
									if ("#text".equals(blockChildNode.getNodeName())) {
										freenetFile.setBlockKey(blockIndex, blockChildNode.getNodeValue());
									}
								}
								blockIndex++;
							}
						}
					}
				}
			}
		}
	}

	public static byte[] transformToByteArray(Document document) {
		ByteArrayOutputStream byteOutput = new ByteArrayOutputStream();
		OutputStreamWriter converter = new OutputStreamWriter(byteOutput, Charset.forName("UTF-8"));
		Result transformResult = new StreamResult(converter);
		Source documentSource = new DOMSource(document);
		try {
			Transformer transformer = getTransformerFactory().newTransformer();
			transformer.transform(documentSource, transformResult);
			byteOutput.close();
			return byteOutput.toByteArray();
		} catch (IOException ioe1) {
		} catch (TransformerConfigurationException tce1) {
		} catch (TransformerException te1) {
		} finally {
			try {
				byteOutput.close();
			} catch (IOException ioe1) {
			}
		}
		return null;
	}

	public static Document transformToDocument(byte[] data) {
		ByteArrayInputStream byteInput = new ByteArrayInputStream(data);
		InputStreamReader converter = new InputStreamReader(byteInput, Charset.forName("UTF-8"));
		Source xmlSource = new StreamSource(converter);
		Result xmlResult = new DOMResult();
		try {
			Transformer transformer = getTransformerFactory().newTransformer();
			transformer.transform(xmlSource, xmlResult);
			return (Document) ((DOMResult) xmlResult).getNode();
		} catch (TransformerConfigurationException tce1) {
		} catch (TransformerException te1) {
		} finally {
			if (byteInput != null)
				try {
					byteInput.close();
				} catch (IOException ioe1) {
				}
			if (converter != null)
				try {
					converter.close();
				} catch (IOException ioe1) {
				}
		}
		return null;
	}

	public static Document parseXML(byte[] data) {
		ByteArrayInputStream dataInput = new ByteArrayInputStream(data);
		try {
			Document newDocument = getDocumentBuilder().parse(dataInput);
			return newDocument;
		} catch (SAXException se1) {
		} catch (IOException ioe1) {
		} finally {
			try {
				dataInput.close();
			} catch (IOException ioe1) {
			}
		}
		return null;
	}

}
