/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freenet.fcp;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * @author David Roden <droden@gmail.com>
 * @version $Id: FcpGenerateCHK.java,v 1.1 2005/11/28 22:54:44 bombe Exp $
 */
public class FcpGenerateCHK extends FcpCommand {

	private InputStream dataInput;
	private InputStream metadataInput;
	private long dataLength;
	private long metadataLength;
	private String uri;

	public FcpGenerateCHK(byte[] data) throws IllegalArgumentException {
		this(data, new byte[0]);
	}

	public FcpGenerateCHK(byte[] data, byte[] metadata) throws IllegalArgumentException {
		this(new ByteArrayInputStream(data), data.length, new ByteArrayInputStream(metadata), metadata.length);
	}
	
	public FcpGenerateCHK(InputStream dataInput, long dataLength, InputStream metadataInput, long metadataLength) {
		this.dataInput = dataInput;
		this.dataLength = dataLength;
		this.metadataInput = metadataInput;
		this.metadataLength = metadataLength;
	}

	/**
	 * @see freenet.fcp.FcpCommand#getName()
	 */
	public String getName() {
		return "GenerateCHK";
	}

	/**
	 * @return Returns the uri.
	 */
	public String getURI() {
		return uri;
	}

	/**
	 * @see freenet.fcp.FcpCommand#writeCommand(java.io.OutputStream)
	 */
	public boolean writeCommand(OutputStream outputStream) throws IOException {
		outputStream.write(("DataLength=" + encodeHex(dataLength + metadataLength) + "\r\n").getBytes());
		if (metadataLength > 0) {
			outputStream.write(("MetadataLength=" + encodeHex(metadataLength) + "\r\n").getBytes());
		}
		outputStream.write("Data\r\n".getBytes());
		copyStream(outputStream, metadataInput, metadataLength);
		copyStream(outputStream, dataInput, dataLength);
		return false;
	}

	/**
	 * @see freenet.fcp.FcpCommand#handleFcpMessage(freenet.fcp.FcpMessage)
	 */
	public void handleFcpMessage(FcpMessage fcpMessage) throws FcpException {
		if (fcpMessage.getName().equals("Success")) {
			uri = fcpMessage.getParameter("URI");
			complete = true;
		}
	}

}
