/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freenet.fcp;

/**
 * This class is merely a container class for the Freenet node.
 * 
 * @author David Roden <droden@gmail.com>
 * @version $Id: FcpNode.java,v 1.1 2005/11/28 22:54:44 bombe Exp $
 */
public class FcpNode {

	/**
	 * The hosename of the Freenet node, defaults to <code>"127.0.0.1"</code>.
	 */
	private String host = "127.0.0.1";

	/**
	 * The Freenet node's FCP port, defaults to <code>8481</code>.
	 */
	private int port = 8481;

	/**
	 * Creates a new FCP node with the default values.
	 */
	public FcpNode() {
	}

	/**
	 * Creates a new FCP node at the specified hostname and default port.
	 * 
	 * @param host
	 *            The hosename of the Freenet node
	 */
	public FcpNode(String host) {
		this.host = host;
	}

	/**
	 * Creates a new FCP node at the specified hostname and port.
	 * 
	 * @param host
	 *            The hostname of the Freenet node
	 * @param port
	 *            The Freenet node's FCP port
	 */
	public FcpNode(String host, int port) {
		this.host = host;
		this.port = port;
	}

	/**
	 * Returns the hostname of this node.
	 * 
	 * @return The hostname of this node
	 */
	public String getHost() {
		return host;
	}

	/**
	 * Sets the hostname of this node.
	 * 
	 * @param host
	 *            The hostname of this node
	 */
	public void setHost(String host) {
		this.host = host;
	}

	/**
	 * Returns the Freenet node's FCP port.
	 * 
	 * @return The Freenet node's FCP port
	 */
	public int getPort() {
		return port;
	}

	/**
	 * Sets the Freenet node's FCP port.
	 * 
	 * @param port
	 *            The Freenet node's FCP port
	 */
	public void setPort(int port) {
		this.port = port;
	}

}
